/* $Id: espda_struct.h 196046 2017-01-30 13:28:09Z gcupani $
 *
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Author: gcupani $
 * $Date: 2017-01-30 13:28:09 +0000 (Mon, 30 Jan 2017) $
 * $Revision: 196046 $
 * $Name:  $
 */

#ifndef ESPDA_STRUCT_H
#define ESPDA_STRUCT_H

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define espda_struct_check(A, B) \
            if (A != NULL) { \
                B; \
            } \

#define espda_struct_copy_old(A, B, C) \
            espda_struct_check(A, cpl_table_copy_data_double(A, C, \
                    cpl_table_get_data_double(A, B))); \


// merge?            
#define espda_struct_extract_old(A, B, C, D, E, F) \
            if (cpl_table_has_column(A, C) == 1) { \
                cpl_table_unselect_all(A); \
                if (D == CPL_TYPE_INT)    { \
                    cpl_table_or_selected_int(A, C, E, F); \
                } \
                else if (D == CPL_TYPE_DOUBLE) { \
                    cpl_table_or_selected_double(A, C, E, F); \
                } \
                if (cpl_table_count_selected(A) > 0) { \
                    B = cpl_table_extract_selected(A); \
                } \
                cpl_table_select_all(A); \
            } \

#define espda_struct_extract_old_2(A, B, C, D) \
            cpl_table_unselect_all(A); \
            cpl_table_or_selected_window(A, C, D); \
            if (cpl_table_count_selected(A) > 0) { \
                B = cpl_table_extract_selected(A); \
            }  \
            cpl_table_select_all(A); \



#define espda_struct_initcol(A, B, C) \
            if (cpl_table_has_column(A, B) != 1) { \
                cpl_table_new_column(A, B, C); \
                cpl_size T = cpl_table_get_nrow(A); \
                if (C == CPL_TYPE_INT) { \
                    cpl_table_fill_column_window_int   (A, B, 0, T, \
                        INV_INT); \
                } \
                else if (C == CPL_TYPE_DOUBLE) { \
                    cpl_table_fill_column_window_double(A, B, 0, T, \
                        INV_DOUBLE); \
                } \
                else if (C == CPL_TYPE_STRING) { \
                    cpl_table_fill_column_window_string(A, B, 0, T, \
                        INV_STRING); \
                } \
            } \

#define espda_struct_insert(A, B, C) \
            if      (A != NULL && B != NULL) { \
                cpl_table_insert(A, B, C); \
            } \
            else if (A == NULL && B != NULL) { \
                A = cpl_table_duplicate(B); \
            }  \
             
#define espda_struct_insert2(A, B, C) \
            if (A) { \
                B = cpl_table_duplicate(C); \
            } \
            else { \
                cpl_table_insert(B, C, cpl_table_get_nrow(B)); \
            } \

#define espda_struct_norm(A, B, C, D, E, F) \
            if (!cpl_table_has_column(A, E)) { \
                cpl_table_new_column(A, E, CPL_TYPE_DOUBLE); \
            } \
            cpl_table_copy_data_double(A, E, \
                cpl_table_get_data_double(A, B)); \
            cpl_table_divide_columns(A, E, D); \
            if (C != NULL && F != NULL) { \
                if (!cpl_table_has_column(A, F)) { \
                    cpl_table_new_column(A, F, CPL_TYPE_DOUBLE); \
                } \
                cpl_table_copy_data_double(A, F, \
                    cpl_table_get_data_double(A, C)); \
                cpl_table_divide_columns(A, F, D); \
            } \
                       
#define espda_struct_startend(A, B, C, D, E) \
            cpl_type TB = cpl_table_get_column_type(A, B); \
            cpl_type TC = cpl_table_get_column_type(A, C); \
            if      (TB == CPL_TYPE_INT)    { \
                D = cpl_table_get_int(A, B, 0, NULL); \
            } \
            else if (TB == CPL_TYPE_DOUBLE) { \
                D = cpl_table_get_double(A, B, 0, NULL); \
            } \
            if      (TC == CPL_TYPE_INT)    { \
                E = cpl_table_get_int(A, C, cpl_table_get_nrow(A) - 1, NULL); \
            } \
            else if (TC == CPL_TYPE_DOUBLE) { \
                E = cpl_table_get_double(A, C, cpl_table_get_nrow(A) - 1, NULL); \
            } \
        
#define espda_struct_size(A, B, C) \
            if (A != NULL) { \
                B = cpl_table_get_nrow(A); \
            } \
            else { \
                B = C; \
            } \
            
#define espda_struct_sort_old(A, B, C) \
            cpl_propertylist *sort = cpl_propertylist_new(); \
            cpl_propertylist_append_bool(sort, B, C); \
            cpl_table_sort(A, sort); \
            cpl_propertylist_delete(sort); \

#define espda_struct_sort_old2(A, B, C, D, E) \
            if (A != NULL) {                                     \
                cpl_propertylist *sort = cpl_propertylist_new(); \
                cpl_propertylist_append_bool(sort, B, C);        \
                cpl_propertylist_append_bool(sort, D, E);        \
                cpl_table_sort(A, sort);                         \
                cpl_propertylist_delete(sort);                   \
            }                                                    \
            
#define espda_struct_window(A, B, C, D, E, F)                       \
            if (A != NULL) {                                        \
                if      (!strcmp(E, "IN"))  {                       \
                    cpl_table *T = NULL;                            \
                    espda_struct_extract_old(A, T, B, CPL_TYPE_DOUBLE,  \
                        CPL_NOT_LESS_THAN,    C);                   \
                    if (T != NULL) { \
                        espda_struct_extract_old(T, F, B, CPL_TYPE_DOUBLE,  \
                            CPL_NOT_GREATER_THAN, D);                   \
                    } \
                    cpl_table_delete(T);                            \
                }                                                   \
                else if (!strcmp(E, "OUT")) {                       \
                    cpl_table *T1 = NULL, *T2 = NULL;               \
                    espda_struct_extract_old(A, T1, B, CPL_TYPE_DOUBLE, \
                        CPL_LESS_THAN,    C);                       \
                    espda_struct_extract_old(A, T2, B, CPL_TYPE_DOUBLE, \
                        CPL_GREATER_THAN, D);                       \
                    if (T2 != NULL) { \
                        espda_struct_insert(F, T2, 0);                  \
                    } \
                    if (T1 != NULL) { \
                        espda_struct_insert(F, T1, 0);                  \
                    } \
                    cpl_table_delete(T1);                           \
                    cpl_table_delete(T2);                           \
                }                                                   \
            }                                                       \


/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "espda_dfs.h"
#include "espda_frame.h"
#include "espda_util.h"

/*-----------------------------------------------------------------------------
                                   Functions prototypes
 -----------------------------------------------------------------------------*/

cpl_error_code  espda_struct_copy  (cpl_table    *,
                                     const char   *, 
                                     cpl_size,
                                     cpl_table    *,
                                     const char   *,
                                     cpl_size);
                                      
cpl_error_code espda_struct_delete(const char *,
                                 int,
                                 ...);

cpl_error_code  espda_struct_erase  (cpl_table    *, 
                                     int,
                                     ...);
                                 
cpl_table      *espda_struct_extract(cpl_table   *,
                                     int, 
                                     ...);

cpl_error_code  espda_struct_init   (cpl_table    *, 
                                     const char   *,
                                     cpl_type);
                                     
cpl_error_code  espda_struct_select (cpl_table    *,
                                     int,
                                     va_list);

cpl_error_code  espda_struct_sort   (cpl_table   *,
                                      int,
                                      ...); 


#endif
