/* 
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Id: espda_voigt.h 173006 2015-05-06 15:44:32Z gcupani $
 * $Author: gcupani $
 * $Date: 2015-05-06 17:44:32 +0200 (Wed, 06 May 2015) $
 * $Revision: 173006 $
 */

#ifndef ESPDA_VOIGT_H
#define ESPDA_VOIGT_H

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <gsl/gsl_eigen.h>
#include <gsl/gsl_multimin.h>
#include <string.h>

#include "espda_dfs.h"
#include "espda_line.h"
//#include "espda_spec.h"  // Needed for the unit test
//#include "espda_model.h"
#include "espda_struct.h"

/*-----------------------------------------------------------------------------
                                   Structures
 -----------------------------------------------------------------------------*/

struct espda_voigt_integ_params{double a;    
                                double u;};

/*-----------------------------------------------------------------------------
                                   Functions prototypes
 -----------------------------------------------------------------------------*/

cpl_error_code espda_voigt_create (cpl_table  	    *,
                                   cpl_array  	   **,
                                   cpl_array  	   **,
                                   gsl_vector 	   **,
                                   cpl_table  	   **,
                                   cpl_size   	    *);

double         espda_voigt_integ  (double,
                          	   void      	    *);

cpl_error_code espda_voigt_merge  (cpl_table  	    *,
                                   cpl_array  	    *,
                                   cpl_array  	    *,
                                   cpl_array  	    *,
                                   const gsl_vector 	    *,
                                   const gsl_matrix 	    *,
                                   cpl_table  	    *,
                                   cpl_array  	   **,
                                   cpl_array  	   **);

cpl_error_code espda_voigt_prep   (cpl_table  	    *,
                                   cpl_table  	    *,
                                   double,
                                   double,
                                   const char 	    *,
                                   //const char 	    *,
                                   cpl_table  	   **);

cpl_error_code espda_voigt_transf (double,
                                   double,
       	                           const double,
       	                           const double,
       	                           const char     *,
                                   double         *,
                                   double         *);

#endif
